<!DOCTYPE html>
<html>
<head>
    <title>
        {% block title %}
            Title
        {% endblock title %}
    </title>
</head>
<body>
    {% block body %}
        <h1>Body</h1>
    {% endblock body %}
</body>
</html>
